using System;
using System.Collections;
using System.Collections.Specialized;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Data.SqlClient;
using System.Security.Permissions;
using System.Web;
using System.Web.UI;

namespace SqlSiteMap
{
    public interface INodeEditable {
        bool Parentable { get; }
        bool Editable { get; }
        bool PropertyChangable { get; }
    }

    public abstract class SiteMapNodeBase : SiteMapNode, INodeEditable {
        private int _id = -1;
        private int _parentId;
        private short _menuOrder;
        private string _iconFile;
        private bool _visible = true;
        private string _classParam;
        private string _instanceParam;

        public abstract bool Parentable { get; }
        public abstract bool Editable { get; }
        public abstract bool PropertyChangable { get; }

        public int Id {
            get { return _id; }
        }
        public string ClassParam {
            get { return _classParam; }
        }
        public string InstanceParam {
            get { return _instanceParam; }
        }
        public int ParentId {
            get { return _parentId; }
            set { _parentId = value; }
        }
        public short MenuOrder {
            get { return _menuOrder; }
            set { _menuOrder = value; }
        }
        public string IconFile {
            get { return _iconFile; }
            set { _iconFile = value; }
        }
        public bool Visible {
            get { return _visible; }
            set { _visible = value; }
        }

        public SiteMapNodeBase(SiteMapProvider provider, 
            string key, string classParam, string instanceParam)
            : base(provider, key) {
                _classParam = classParam;
                _instanceParam = instanceParam;
        }

        public SiteMapNodeBase(SiteMapProvider provider, int id, 
            string key, string classParam, string instanceParam) 
            : base(provider, key) {
                _id = id;
                _classParam = classParam;
                _instanceParam = instanceParam;
        }

        protected SiteMapProviderBase GetProvider() {
            return (SiteMapProviderBase)this.Provider;
        }
    }

    public class SiteMapNodeBaseComparer : IComparer<SiteMapNodeBase> {
        public int Compare(SiteMapNodeBase x, SiteMapNodeBase y)
        {
            return x.MenuOrder.CompareTo(y.MenuOrder);
        }
    }

    public class GeneralSiteMapNode : SiteMapNodeBase {
        public override bool Parentable { 
            get { return true; } 
        }
        public override bool PropertyChangable {
            get { return true; }
        }
        private bool _editable = true;
        public override bool Editable {
            get { return _editable; }
        }

        internal void SetEditable(bool editable) {
            _editable = editable;
        }

        public GeneralSiteMapNode(SiteMapProvider provider, int id, 
            string key, string classParam, string instanceParam)
            : base(provider, id, key, classParam, instanceParam) { }
    }

    public class BoardSiteMapNode : SiteMapNodeBase {
        private bool _available;
        private bool _initialized = false;
        private int _boardNo = -1;
        private string _dataTable = null;
        private int _maximumChildNodes = 10;
        private SiteMapNodeCollection _childNodes = null;
        private string _contentPagePath = null;
        private bool _editable = true;

        private ChildNodesDisplayMode _childNodesDisplayMode = ChildNodesDisplayMode.None;

        public enum ChildNodesDisplayMode { 
            None,
            SupportSiteMapPath,
            EnumerateContentTitle
        }

        public bool Available {
            get { return _available; }
        }

        public override string Title {
            get { return base.Title; }
            set { throw new NotSupportedException("Title Ӽ    ϴ."); }
        }

        public override string Url {
            get { return base.Url; }
            set { throw new NotSupportedException("Url Ӽ    ϴ."); }
        }

        public override bool Parentable {
            get { return false; }
        }

        public override bool PropertyChangable {
            get { return false; }
        }

        public override bool Editable {
            get { return _editable; }
        }

        internal void SetEditable(bool editable) {
            _editable = editable;
        }

        public BoardSiteMapNode(SiteMapProvider provider, int id, 
            string key, string classParam, string instanceParam)
            : base(provider, id, key, classParam, instanceParam) {
            SiteMapProviderBase prov = GetProvider();
            string val;
            val = prov.ConfigAttributes["boardSiteMapNode_ChildNodesDisplayMode"];
            SetChildNodesDisplayMode(val);
            val = prov.ConfigAttributes["boardSiteMapNode_MaximumChildNodes"];
            int maximumChildNodes;
            if (int.TryParse(val, out maximumChildNodes))
                _maximumChildNodes = maximumChildNodes;

            ParseParam();
            FillProperties();

            CreateChildNodes();
        }

        private void CreateChildNodes() {
            if (_childNodesDisplayMode == ChildNodesDisplayMode.None)
                return;

            SiteMapProviderBase prov = GetProvider();
            _contentPagePath = prov.BoardRelativeDirectory 
                + Definations.CntentPageName;
            string pathAndQuery = _contentPagePath + "?" 
                + Definations.BoardNoParmName + "=" + _boardNo;
            
            GeneralSiteMapNode node = new GeneralSiteMapNode(this.Provider,
                -this.Id, this.Id.ToString("00000") + "_" + _boardNo.ToString("00000"), null, null);
            node.Title = "dummy_" + _boardNo;
            node.Url = pathAndQuery;
            node.SetEditable(false);
            
            GetProvider().AddNodeWithProvider(node, this);
            if (_childNodesDisplayMode == ChildNodesDisplayMode.SupportSiteMapPath) {
                //_childNodes = new SiteMapNodeCollection(1);
                //_childNodes.Add(node);
            }
            else {
                _childNodes = new SiteMapNodeCollection(_maximumChildNodes);
            }
        }

        private void SetChildNodesDisplayMode(string modeStr) {
            if (String.IsNullOrEmpty(modeStr))
                return;
            switch (modeStr) {
                case "None":
                    _childNodesDisplayMode = ChildNodesDisplayMode.None;
                    break;
                case "SupportSiteMapPath":
                    _childNodesDisplayMode = ChildNodesDisplayMode.SupportSiteMapPath;
                    break;
                case "EnumerateContentTitle":
                    _childNodesDisplayMode = ChildNodesDisplayMode.EnumerateContentTitle;
                    break;
            }
        }

        private void ParseParam() {
            if (!string.IsNullOrEmpty(this.InstanceParam)) {
                foreach (string parm in this.InstanceParam.Split(new char[] { ';' })) {
                    string[] p = parm.Split(new char[] { '=' });
                    if (p.Length != 2 || string.IsNullOrEmpty(p[0]))
                        continue;
                    switch (p[0]) {
                        case "boardNo":
                            if (!int.TryParse(p[1], out _boardNo))
                                throw new InvalidOperationException("BoardSiteMapNode Ϸ SiteMap ̺ nodeObjectParam ʵ忡 boardNo Ķ͸ ؾմϴ.");
                            break;
                        case "childNodesDisplayMode":
                            SetChildNodesDisplayMode(p[1]);
                            break;
                        case "maximumChildNodes":
                            _maximumChildNodes = int.Parse(p[1]);
                            break;
                    }
                }
            }
        }

        private void FillProperties() {
            if (_initialized)
                return;
            SiteMapProviderBase prov = GetProvider();
            string connStr = ConfigurationManager.ConnectionStrings[prov.ConnectionStringName].ConnectionString;
            SqlConnection conn = new SqlConnection(connStr);
            SqlCommand cmd = new SqlCommand("SM_GetBoardInfoByNo", conn);
            cmd.CommandType = CommandType.StoredProcedure;
            cmd.Parameters.Add(new SqlParameter("@no", SqlDbType.Int)).Value = _boardNo;

            conn.Open();
            SqlDataReader dr = cmd.ExecuteReader();
            if (dr.Read())  {
                base.Url = prov.BoardRelativeDirectory + Definations.ListPageName + "?board=" + _boardNo;
                base.Title = (string)dr["subject"];
                _dataTable = (string)dr["tableName"];
                _available = true;
            }
            else {
                _available = false;
                base.Title = "<i> Խ ã  ϴ.</i>(id=" + this.Id + ")";
                ArrayList roles = new ArrayList(1);
                roles.Add("*");
                base.Roles = roles;
            }
            dr.Close();
            conn.Close();

            _initialized = true;
        }

        public override SiteMapNodeCollection ChildNodes {
            get {
                if (_childNodesDisplayMode == ChildNodesDisplayMode.None)
                    return base.ChildNodes;

                if (_childNodesDisplayMode == ChildNodesDisplayMode.SupportSiteMapPath)
                    return _childNodes;
                else {
                    _childNodes.Clear();

                    string pathAndQuery = _contentPagePath + "?"
                        + Definations.BoardNoParmName + "=" + _boardNo;
                    string connStr = ConfigurationManager.ConnectionStrings[
                        GetProvider().ConnectionStringName].ConnectionString;
                    SqlConnection conn = new SqlConnection(connStr);
                    SqlCommand sc =
                        new SqlCommand(" SELECT top " + _maximumChildNodes + " * " +
                                        " FROM " + _dataTable + " " +
                                        " ORDER BY regDate desc",
                                        conn);

                    SqlDataReader sdr = null;
                    try {
                        conn.Open();
                        sdr = sc.ExecuteReader();

                        GeneralSiteMapNode node;
                        while (sdr.Read()) {
                            int no = (int)sdr["no"];
                            node = new GeneralSiteMapNode(this.Provider,
                                no, no.ToString("000000"), null, null);
                            node.Title = (string)sdr["title"] + "(" + (string)sdr["writerName"] + ")";
                            node.Url = pathAndQuery + "&" + Definations.ContentNoParmName 
                                + "=" + no;
                            node.SetEditable(false);

                            _childNodes.Add(node);
                        }
                    }
                    finally {
                        if (sdr != null)
                            sdr.Close();
                        conn.Close();
                    }

                    return SiteMapNodeCollection.ReadOnly(_childNodes);
                }
            }
            set {
                throw new InvalidOperationException("BoardSiteMapNode ڽĳ带    ϴ.");
            }
        }
    }
}
