using System;
using System.Collections;
using System.Collections.Specialized;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Data.SqlClient;
using System.Security.Permissions;
using System.Web;
using System.Web.UI;

namespace SqlSiteMap
{
    public abstract class SiteMapProviderBase : StaticSiteMapProvider {
        protected SiteMapNodeBase _rootNode = null;
        protected string _connStrName = null;

        private NameValueCollection _configAttributes;
        private string _boardRelativeDirectory = null;
        private string _contentPagePath = null;

        public NameValueCollection ConfigAttributes {
            get { return _configAttributes; }
        }
        public string BoardRelativeDirectory {
            get { return _boardRelativeDirectory; }
            set { _boardRelativeDirectory = value; }
        }
        internal void UpdateSiteMap() {
            _rootNode = null;
            BuildSiteMap();
        }
        internal string ConnectionStringName {
            get { return _connStrName; }
        }

        public override void Initialize(string name, NameValueCollection attributes) {
            base.Initialize(name, attributes);

            string boardRelativeDirectory = attributes["boardRelativeDirectory"];
            if (string.IsNullOrEmpty(boardRelativeDirectory))
                throw new ConfigurationErrorsException("GeneralSiteMapProvider ι̴ Ͽ boardRelativeDirectory Ӽ ؾ մϴ.");
            _boardRelativeDirectory = boardRelativeDirectory;
            attributes.Remove("boardRelativeDirectory");

            _configAttributes = attributes;
        }

        internal void AddNodeWithProvider(SiteMapNode node, SiteMapNode parentNode)
        {
            base.AddNode(node, parentNode);
        }

        public override SiteMapNode CurrentNode {
            get {
                HttpContext context = HttpContext.Current;
                if (_contentPagePath == null) {
                    _contentPagePath = _boardRelativeDirectory + Definations.CntentPageName;

                    Page page = (context.CurrentHandler as Page);
                    _contentPagePath = page.ResolveUrl(_contentPagePath).ToLower();
                }

                string searchPath = context.Request.FilePath;
                if (_contentPagePath.Equals(searchPath.ToLower())) {
                    searchPath += "?" + Definations.BoardNoParmName + "=" 
                        + context.Request.QueryString[Definations.BoardNoParmName];
                    SiteMapNode node = this.FindSiteMapNode(searchPath);
                    if (node != null && node.ParentNode is BoardSiteMapNode) {
                        SiteMapNode changedNode = this.ResolveSiteMapNode(context);
                        if (changedNode != null)
                            node = changedNode;

                        if (node.IsAccessibleToUser(context))
                            return node;
                        else 
                            return null;
                    }
                }

                return base.CurrentNode;
            }
        }
    }

    public class GeneralSiteMapProvider : SiteMapProviderBase {
        private string _connStr = null;
        private bool _initialized = false;
        private bool _isNotEditMode = true;
        private bool _allowAllUserIfEmptyUrl = false;

        public GeneralSiteMapProvider() { }

        public GeneralSiteMapProvider(bool notEditable) {
            _isNotEditMode = notEditable;
        }

        protected override SiteMapNode GetRootNodeCore() {
            return BuildSiteMap();
        }

        public override void Initialize(string name, 
            NameValueCollection attributes) {
            if (_initialized)
                throw new InvalidOperationException("ʱȭ ι ̻   ϴ.");

            _connStrName = attributes["connectionStringName"];
            if (string.IsNullOrEmpty(_connStrName))
                throw new ConfigurationErrorsException("ι̴  connectionStringName Ӽ  ʾҽϴ");
            attributes.Remove("connectionStringName");
            _connStr = ConfigurationManager.ConnectionStrings[_connStrName].ConnectionString;

            string allowAllUserIfEmptyUrl = attributes["allowAllUserIfEmptyUrl"];
            if (!string.IsNullOrEmpty(allowAllUserIfEmptyUrl)) {
                _allowAllUserIfEmptyUrl = allowAllUserIfEmptyUrl.ToLower().Equals("true");
                attributes.Remove("allowAllUserIfEmptyUrl");
            }

            base.Initialize(name, attributes);
            _initialized = true;
        }

        protected override void Clear() {
            lock (this) {
                _rootNode = null;
                base.Clear();
            }
        }

        public override SiteMapNode BuildSiteMap() {
            lock (this) {
                if (!_initialized)
                    throw new Exception("ʱȭ  BuildSiteMap ޼带 ȣ Ҽ ϴ.");

                if (_rootNode != null)
                    return _rootNode;

                Clear();

                int rootNodeId = -1;
                SqlConnection conn = new SqlConnection(_connStr);
                SqlCommand cmd = new SqlCommand("SM_GetRootNode", conn);
                cmd.CommandType = CommandType.StoredProcedure;
                cmd.Parameters.Add("@checkVisible", SqlDbType.Bit).Value = _isNotEditMode;

                conn.Open();
                SqlDataReader dr = cmd.ExecuteReader();
                if (dr.Read()) {
                    rootNodeId = (int)dr["id"];
                    string url = (string)dr["url"];
                    string key = rootNodeId.ToString();
                    string classParam = dr["classParam"] == DBNull.Value ? 
                        null : (string)dr["classParam"];
                    string instanceParam = dr["nodeObjectParam"] == DBNull.Value ?
                        null : (string)dr["nodeObjectParam"];

                    _rootNode = CreateSiteMapNodeObject((string)dr["objectName"],
                        rootNodeId, key, classParam, instanceParam);

                    _rootNode.Url = url;
                    if (_rootNode.Url.Equals(string.Empty))
                        _rootNode.Url = null;
                    _rootNode.Title = (string)dr["title"];
                    _rootNode.Description = dr["description"] == DBNull.Value 
                        ? "" : (string)dr["description"];
                    _rootNode.IconFile = dr["iconFile"] == DBNull.Value 
                        ? "" : (string)dr["iconFile"];
                    _rootNode.MenuOrder = (short)dr["menuOrder"];
                    _rootNode.ParentId = (int)dr["parentId"];
                    if (dr["roles"] != DBNull.Value) {
                        ArrayList roles = new ArrayList(5);
                        foreach (string role in ((string)dr["roles"]).Split(new char[] { ',' }))
                            if (!role.Trim().Equals(string.Empty))
                                roles.Add(role);
                        _rootNode.Roles = roles;
                    }
                    if (_allowAllUserIfEmptyUrl
                        && (_rootNode.Roles == null 
                           || _rootNode.Roles.Count == 0)) {
                            ArrayList roles = new ArrayList(1);
                            roles.Add("*");
                            _rootNode.Roles = roles;
                    }
                    _rootNode.Visible = (bool)dr["visible"];
                    _rootNode.ReadOnly = _isNotEditMode;
                    AddNode(_rootNode);
                }
                dr.Close();
                conn.Close();

                if (_rootNode != null)
                    AddChildren(_rootNode, rootNodeId);

                return _rootNode;
            }
        }

        private SiteMapNodeBase CreateSiteMapNodeObject(
            string typeName, int id, string key, 
            string classParam, string instanceParam) {
            Type type = Type.GetType(typeName, true);
            object[] paramArray = new object[5];
            paramArray[0] = this;
            paramArray[1] = id;
            paramArray[2] = key;
            paramArray[3] = classParam;
            paramArray[4] = instanceParam;

            return (SiteMapNodeBase)Activator.CreateInstance(type, paramArray);
        }

        private void AddChildren(
            SiteMapNodeBase parentNode, int parentID) {
            if (!parentNode.Parentable)
                return;

            SqlConnection conn = new SqlConnection(_connStr);
            SqlCommand cmd = new SqlCommand("SM_GetNodeByParentId");
            SqlDataAdapter adapter = new SqlDataAdapter();

            cmd.CommandType = CommandType.StoredProcedure;
            cmd.Connection = conn;
            cmd.Parameters.Add("@parentId", SqlDbType.Int).Value = parentID;
            cmd.Parameters.Add("@checkVisible", SqlDbType.Bit).Value = _isNotEditMode;
            
            adapter.SelectCommand = cmd;

            DataTable dt = new DataTable();
            adapter.Fill(dt);
            foreach (DataRow row in dt.Rows) {
                int id = (int)row["id"];
                string url = row["url"] == DBNull.Value
                    ? null : (string)row["url"];
                string key = id.ToString();
                string classParam = row["classParam"] == DBNull.Value 
                    ? null : (string)row["classParam"];
                string instanceParam = row["nodeObjectParam"] == DBNull.Value
                    ? null : (string)row["nodeObjectParam"];
                SiteMapNodeBase node = CreateSiteMapNodeObject(
                    (string)row["objectName"],
                    id, key, classParam, instanceParam);

                if (node.PropertyChangable && string.IsNullOrEmpty(node.Url)) {
                    node.Url = url;
                    if (node.Url.Equals(string.Empty))
                        node.Url = null;
                }
                if (node.PropertyChangable && string.IsNullOrEmpty(node.Title)) 
                    node.Title = row["title"] == DBNull.Value 
                        ? null : (string)row["title"];
                if (node.PropertyChangable && string.IsNullOrEmpty(node.Description)) 
                    node.Description = row["description"] == DBNull.Value 
                        ? "" : (string)row["description"];
                if (string.IsNullOrEmpty(node.IconFile))
                    node.IconFile = row["iconFile"] == DBNull.Value
                        ? "" : (string)row["iconFile"];
                node.MenuOrder = (short)row["menuOrder"];
                node.ParentId = (int)row["parentId"];
                if (row["roles"] != DBNull.Value) {
                    ArrayList roles = new ArrayList(5);
                    foreach (string role in ((string)row["roles"]).Split(new char[] { ',' }))
                        if (!role.Trim().Equals(string.Empty))
                            roles.Add(role);
                    node.Roles = roles;
                }
                if (_allowAllUserIfEmptyUrl
                    && (node.Roles == null
                       || node.Roles.Count == 0)) {
                    ArrayList roles = new ArrayList(1);
                    roles.Add("*");
                    node.Roles = roles;
                }
                node.Visible = (bool)row["visible"];
                node.ReadOnly = _isNotEditMode;

                AddNode(node, parentNode);
                AddChildren(node, id);
            }
        }
    }
}
